SchedHeuristic

Enum SchedHeuristic 

pub enum SchedHeuristic {
    None,
    ConnectBind,
    Random,
    StickyRandom,
}
Apply a specialized scheduling heuristic which may help exercise certain bugs.

Variants



None

Don’t modify the scheduling algorithm.



ConnectBind

Prioritize connect and deprioritize bind to exercise races



Random

Random: Randomly pick any available thread to make progress.



StickyRandom

Sticky Random: Randomly pick any available thread. On the next round, and after the thread is parked, randomly choose if we will continue executing on the same thread, or picking another one.