Types and parsers shared by procfs producers and consumers.
Structs
- Mount
Info Row - Identity-bearing fields from one Linux
/proc/*/mountinforow.
Constants
Functions
- mount_
ids_ are_ ordered_ subset - Whether every visible mount ID occurs once and in the same relative order as the producer-captured namespace.
- parse_
fdinfo_ mount_ id - Parse the one decimal
mnt_idfield required by Linux fdinfo. - parse_
mountinfo - Strictly parse a Linux
/proc/*/mountinfosnapshot.