Module schedule

Module schedule 

Structs

SchedEvent
A scheduled action by one thread in the system. This can be recorded, or replayed to guide the schedule.
SigWrapper
A signal the scheduler can name, INCLUDING the realtime signals nix cannot.

Enums

Op
NOTE [Event Semantics]
SyscallPhase
Which phase of the syscall did we observe on a given event: the prehook or the posthook.

Type Aliases

InstructionPointer
The type of the RIP value.