pub enum SyscallPhase {
Prehook,
Polling,
Posthook,
}Expand description
Which phase of the syscall did we observe on a given event: the prehook or the posthook.
Variants
Prehook
The event was recorded before physically beginning the syscall.
Polling
An internal (nonblocking) retry of the syscall to check if its done yet (but it wasn’t).
Posthook
The event was recorded after the syscall logically completed.