pub struct SchedEvent {
pub dettid: DetTid,
pub op: Op,
pub count: u32,
pub start_rip: Option<InstructionPointer>,
pub end_rip: Option<InstructionPointer>,
pub end_time: Option<LogicalTime>,
}Expand description
A scheduled action by one thread in the system. This can be recorded, or replayed to guide the schedule.
Fields
dettid: DetTidThe thread that originated the event.
op: OpThe operation performed by the thread.
count: u32The consecutive count of that same operation (run length encoding).
start_rip: Option<InstructionPointer>The instruction pointer before this batch of operations.
end_rip: Option<InstructionPointer>The instruction pointer after this batch of operations.
end_time: Option<LogicalTime>An optional snapshot of the thread logical time at this point. This includes time waiting on the global scheduler.
Implementations
Sourceimpl SchedEvent
impl SchedEvent
Sourcepub fn syscall(dettid: DetTid, sysno: Sysno, phase: SyscallPhase) -> SchedEvent
pub fn syscall(dettid: DetTid, sysno: Sysno, phase: SyscallPhase) -> SchedEvent
Add a syscall to the global scheduling history. This takes the instruction pointer of the syscall itself.
Sourcepub fn branches(dettid: DetTid, count: u32) -> SchedEvent
pub fn branches(dettid: DetTid, count: u32) -> SchedEvent
Add a batch of branches to the global scheduling history.
Sourcepub fn with_time(self, time: LogicalDuration) -> SchedEvent
pub fn with_time(self, time: LogicalDuration) -> SchedEvent
Set the logical time directly.
Sourcepub fn with_dettime(self, dt: &DetTime) -> SchedEvent
pub fn with_dettime(self, dt: &DetTime) -> SchedEvent
Correctly set logical time based on the threads current time.
Sourcepub fn with_start_rip(self, start_rip: InstructionPointer) -> Self
pub fn with_start_rip(self, start_rip: InstructionPointer) -> Self
Set the start_rip field. The instruction pointer before the event began executing.
Sourcepub fn with_end_rip(self, end_rip: InstructionPointer) -> Self
pub fn with_end_rip(self, end_rip: InstructionPointer) -> Self
Set the end_rip field. The instruction pointer after the event completed.