pub struct DetTime { /* private fields */ }Expand description
A determinstic notion of time, measuring progress of one thread.
It is based on counting syscalls and conditionals. Ideally it would count instructions, but that is not possible deterministically.
DetTime is a measure of the LOCAL progress of a thread. The notion of
global time is defined by aggretion of multiple local times (vector
clocks, as in the Kendo algorithm).
Here are a few relevant definitions for deterministic time:
Granularity
How rapidly and consistently does the clock tick with thread progress? A finer notion of deterministic time counts more events (ideally instructions). A coarser notion of deterministic time counts fewer events (like syscalls).
Productivity
In corecursion, or coinductive datatypes like streams, productivity means you can get the next result with finite work. Deterministic time can be viewed a stream of “tick events”. We don’t want a guest thread to be able to do an unbounded amount of work, without a tick occurring. For example, retired branches are a safe bet (any non-trivial amount of work will execute a branch), but system calls are not: a spinning thread can burn cycles forever without executing a syscall.
Implementations
Sourceimpl DetTime
impl DetTime
Sourcepub fn clone_for_child(&self) -> Self
pub fn clone_for_child(&self) -> Self
Inherit an absolute clock for a newly created thread without attributing
its ancestors’ work to that thread. Ordinary Clone still copies an
existing clock, including its original inheritance, for RPC snapshots.
Sourcepub fn inherited_nanos(&self) -> LogicalDuration
pub fn inherited_nanos(&self) -> LogicalDuration
Local elapsed time already accounted for by this thread’s ancestors.
Sourcepub fn add_syscall(&mut self)
pub fn add_syscall(&mut self)
Register that another syscall has executed.
Sourcepub fn add_syscall_with_cost(&mut self, nanos: u64)
pub fn add_syscall_with_cost(&mut self, nanos: u64)
Register a syscall with its unscaled virtual-time cost in nanoseconds.
Sourcepub fn add_rcbs_with_multiplier(
&mut self,
count: u64,
factor: RcbTimeMultiplier,
)
pub fn add_rcbs_with_multiplier( &mut self, count: u64, factor: RcbTimeMultiplier, )
Add RCB progress using a deterministic per-thread virtual-time multiplier.
The Q32 accumulator makes the result independent of whether a backend reports the same RCB total in one update or several smaller updates.
Sourcepub fn advance_to(&mut self, deadline: LogicalTime)
pub fn advance_to(&mut self, deadline: LogicalTime)
Advance this local clock to an explicit virtual deadline.
Sourcepub fn as_nanos(&self) -> LogicalTime
pub fn as_nanos(&self) -> LogicalTime
Project deterministic logical time into a rough number of nanoseconds.
Sourcepub fn without_starting(&self) -> LogicalDuration
pub fn without_starting(&self) -> LogicalDuration
Same as as_nanos but without the starting time.
Sourcepub fn user_cpu_time(&self) -> LogicalDuration
pub fn user_cpu_time(&self) -> LogicalDuration
Guest-execution time that corresponds to user-space instructions.
Sourcepub fn system_cpu_time(&self) -> LogicalDuration
pub fn system_cpu_time(&self) -> LogicalDuration
Synthetic time charged for intercepted syscall execution.
Sourcepub fn as_micros(&self) -> Microseconds
pub fn as_micros(&self) -> Microseconds
Project deterministic logical time into a rough number of microseconds.
Sourcepub fn with_multiplier(self, m: f64) -> Self
pub fn with_multiplier(self, m: f64) -> Self
Set the clock multiplier
Sourcepub fn as_duration(&self) -> Duration
pub fn as_duration(&self) -> Duration
Project deterministic time duration from imaginary starting point of deterministic time creation